//
//

#include "stdafx.h"
#include <malloc.h>
#include <math.h>
#include <limits.h> 

//---------------------------------------------------------------------------
//
//---------------------------------------------------------------------------
//
//
//
//
//

#define minVal(a, b)  (((a) < (b)) ? (a) : (b))
#define maxVal(a, b)  (((a) > (b)) ? (a) : (b))
#define fabs(a)       (((a) > 0) ? (a) : (-a))

#define TINY 1.0e-20 

void   sort2arrays(double*,double*,int);
void   calculateRank(double*,double&,int);
double betai(const double, const double, const double);
double betacf(const double, const double, const double);
double gammln(const double);
double erfcc(const double);

//
//
//
//
//

__declspec(dllexport) double __stdcall Pearson(const double *arrayA, int isSeriesA, double *arrayB, int isSeriesB, int bars, int shift, int length, int result)
{
	int n=minVal(bars-shift,length);
		if (bars <= 5) return(-1);
		if (n    <= 1) return(-1);

	//
	//
	//
	//
	//

	int shiftA = shift; if (!isSeriesA) shiftA=(bars-shift-1);
	int shiftB = shift; if (!isSeriesB) shiftB=(bars-shift-1);
	double yt,xt;
	double	syy=0.0,
			sxy=0.0,
			sxx=0.0,
			ay=0.0,
			ax=0.0;

	//
	//
	//
	//
	//

		for (int j=0; j<n; j++)
		{
			if (isSeriesA)
					ax += arrayA[shiftA+j];
			else	ax += arrayA[shiftA-j];
			if (isSeriesB)
					ay += arrayB[shiftB+j];
			else	ay += arrayB[shiftB-j];
		}
		ax /= n; ay /= n;

	//
	//
	// Compute the correlation coefficient.
	//
	//

		for (int j=0; j<n; j++)
		{
			if (isSeriesA)
					xt = arrayA[shiftA+j]-ax;
			else	xt = arrayA[shiftA-j]-ax;
			if (isSeriesB)
					yt = arrayB[shiftB+j]-ay;
			else	yt = arrayB[shiftB-j]-ay;
			sxx += xt*xt;
			syy += yt*yt;
			sxy += xt*yt;
		}
	
		double r=sxy/(sqrt(sxx*syy)+TINY);	

	//
	//
	//
	//
	//

	result = minVal(maxVal(result,0),2);
	switch(result)
	{
		case 0: return(r);
		case 1: 
			{
				double z=0.5*log((1.0+r+TINY)/(1.0-r+TINY)); // Fishers z transformation.
				return(z);
			}
		case 2: 
			{
				double df=n-2;
				double t=r*sqrt(df/((1.0-r+TINY)*(1.0+r+TINY))); // Equation (14.5.5).
				double prob = betai(0.5*df,0.5,df/(df+t*t));     //Students t probability.
				return(prob);
			}
	}	
	return(r);
}

//---------------------------------------------------------------------------
//
//---------------------------------------------------------------------------
//
//
//
//
//

__declspec(dllexport) double __stdcall Kendall(const double *arrayA, int isSeriesA, double *arrayB, int isSeriesB, int bars, int shift, int length, int result)
{
	int n=minVal(bars-shift,length);
		if (bars <= 5) return(-1);
		if (n    <= 1) return(-1);


	//
	//
	//
	//
	//

	int shiftA = shift; if (!isSeriesA) shiftA=(bars-shift-1);
	int shiftB = shift; if (!isSeriesB) shiftB=(bars-shift-1);

	int is=0,n2=0,n1=0;
	double aa,a2,a1;

	//
	//
	//
	//
	//

	for (int j=0; j<n-1;j++) 
	{
		for (int k=j+1; k<n; k++)
		{
			if (isSeriesA)
					a1=arrayA[shiftA+j]-arrayA[shiftA+k];
			else	a1=arrayA[shiftA-j]-arrayA[shiftA-k];
			if (isSeriesB)
					a2=arrayB[shiftA+j]-arrayB[shiftA+k];
			else	a2=arrayB[shiftA-j]-arrayB[shiftA-k];
			aa=a1*a2;
			if (aa != 0.0) 
			{
				++n1;
				++n2;
				aa > 0.0 ? ++is : --is;
			}
			else 
			{
				if (a1 != 0.0) ++n1;
				if (a2 != 0.0) ++n2;
			}
		}
	}
	double tau=is/(sqrt(double(n1))*sqrt(double(n2)));

	//
	//
	//
	//
	//

	result = minVal(maxVal(result,0),3);
	switch(result)
	{
		case 0: return(tau);
		case 1: 
			{
				double z=0.5*log((1.0+tau+TINY)/(1.0-tau+TINY)); // Fishers z transformation
				return(z);
			}
		case 2: 
			{
				double svar = (4.0*n+10.0)/(9.0*n*(n-1.0));
				double z    = tau/sqrt(svar);
				double prob = erfcc(fabs(z)/1.4142136);
				//				return(tau);
				return(prob);
			}
		case 3: 
			{
				double svar = (4.0*n+10.0)/(9.0*n*(n-1.0));
				double z    = tau/sqrt(svar);
				return(z);
			}
	}	
	return(tau);
}


//---------------------------------------------------------------------------
//
//---------------------------------------------------------------------------
//
//
//
//
//

__declspec(dllexport) double __stdcall Spearman(const double *arrayA, int isSeriesA, double *arrayB, int isSeriesB, int bars, int shift, int length, int result)
{
	int n=minVal(bars-shift,length);
		if (bars <= 5) return(0);
		if (n    <= 1) return(0);

	//
	//
	//
	//
	//

	int shiftA = shift; if (!isSeriesA) shiftA=(bars-shift-1);
	int shiftB = shift; if (!isSeriesB) shiftB=(bars-shift-1);

	int size = sizeof(double) * n;
		double* wksp1 = (double *)malloc(size); if (wksp1 == NULL) return(0);
		double* wksp2 = (double *)malloc(size); if (wksp2 == NULL) { free(wksp1); return(0); }

	//
	//
	//
	//
	//

		for (int j=0; j<n; j++)
		{
			if (isSeriesA)
					wksp1[j] = arrayA[shiftA+j];
			else	wksp1[j] = arrayA[shiftA-j];
			if (isSeriesB)
					wksp2[j] = arrayB[shiftB+j];
			else	wksp2[j] = arrayB[shiftB-j];
		}
		double sf; sort2arrays(wksp1,wksp2,n); calculateRank(wksp1,sf,n);
		double sg; sort2arrays(wksp2,wksp1,n); calculateRank(wksp2,sg,n);

		//
		//
		//
		//
		//

		double d=0.0,diff;
			for (int j=0; j<n; j++)
			{
				diff = (wksp1[j]-wksp2[j]);
				d    += diff*diff;
			}
		double en   = n;
		double en3n = en*en*en-en;
		double aved  = en3n/6.0-(sf+sg)/12.0; // Expectation value of D,

		double fac  = (1.0-sf/en3n)*(1.0-sg/en3n);
		double rank = (fac!=0 ? (1.0-(6.0/en3n)*(d+(sf+sg)/12.0))/sqrt(fac):0); // Spearman's rank
		double vard = ((en-1.0)*en*en*(en+1.0)*(en+1.0)/36.0)*fac;              // variance od differences
		double zd   = (d-aved)/sqrt(vard);                                      // number of standars deviations

		//
		//
		//
		//
		//

		free(wksp1); free(wksp2);

	//
	//
	//
	//
	//

	result = minVal(maxVal(result,0),5);
	switch(result)
	{
		case 0: return(rank);
		case 1: 
			{
				//
				//
				// Fishers z transformation
				//
				//

				double z = (rank !=0 ? 0.5*log((1.0+rank+TINY)/(1.0-rank+TINY)) : 0);
				return(z);
			}
		case 2: 
			{
				double prob = 0;
				double fac=(rank+1.0)*(1.0-rank);
				if (fac > 0.0) 
				{
						double t    = rank*sqrt((en-2.0)/fac); // and its t-value,
						double df   = en-2.0;
						       prob = betai(0.5*df,0.5,df/(df+t*t)); // give its p-value.
				}
				return(prob);
			}
		case 3: 
			{
				return(d);
			}
		case 4: 
			{
				return(aved);
			}
		case 5: 
			{
				return(zd);
			}
	}	
	return(d);
}

//
//
//
//
//

inline void SWAP(double &a, double &b) {double dum=a; a=b; b=dum;}
void sort2arrays(double *arr, double *brr, int size)
{
	const int M=7,NSTACK=50;

	//
	//
	//
	//
	//

	double a,b;
	int    n=size;
	int    istack[NSTACK];
	int    i,ir,j,k,jstack=-1,l=0;

	for(int i=0; i<NSTACK; i++) istack[i] = 0;
	ir=n-1;
	for (;;)
	{
		if (ir-l < M)
		{
			for (j=l+1;j<=ir;j++)
			{
				a=arr[j];
				b=brr[j];
				for (i=j-1;i>=l;i--)
				{
					if (arr[i] <= a) break;
						arr[i+1]=arr[i];
						brr[i+1]=brr[i];
				}
				arr[i+1]=a;
				brr[i+1]=b;
			}
			if (jstack < 0) break;
			ir = istack[jstack--];
			l  = istack[jstack--];
		}
		else
		{
			k=(l+ir) >> 1;
			SWAP(arr[k],arr[l+1]);
			SWAP(brr[k],brr[l+1]);
			if (arr[l] > arr[ir])
			{
				SWAP(arr[l],arr[ir]);
				SWAP(brr[l],brr[ir]);
			}
			if (arr[l+1] > arr[ir])
			{
				SWAP(arr[l+1],arr[ir]);
				SWAP(brr[l+1],brr[ir]);
			}
			if (arr[l] > arr[l+1])
			{
				SWAP(arr[l],arr[l+1]);
				SWAP(brr[l],brr[l+1]);
			}
			i=l+1;
			j=ir;
			a=arr[l+1];
			b=brr[l+1];
			for (;;) {
				do i++; while (arr[i] < a);
				do j--; while (arr[j] > a);
				if (j < i) break;
				SWAP(arr[i],arr[j]);
				SWAP(brr[i],brr[j]);
			}
			arr[l+1]=arr[j];
			arr[j]=a;
			brr[l+1]=brr[j];
			brr[j]=b;
			jstack += 2;
//			if (jstack >= NSTACK) nrerror("NSTACK too small in sort2.");
			if (jstack >= NSTACK) break;
			if (ir-i+1 >= j-l)
			{
				istack[jstack]   = ir;
				istack[jstack-1] = i;
				ir=j-1;
			}
			else
			{
				istack[jstack]=j-1;
				istack[jstack-1]=l;
				l=i;
			}
		}
	}
} 

//
//
//
//
//
void calculateRank(double *w, double &s, int n)
{
	int j=1,ji,jt;
	double t,rank;

	s=0.0;
	while (j < n)
	{
		if (w[j] != w[j-1])
		{
			w[j-1]=j;
			++j;
		}
		else
		{
			for (jt=j+1;jt<=n && w[jt-1]==w[j-1];jt++);
			rank=0.5*(j+jt-1);
			for (ji=j;ji<=(jt-1);ji++)
				w[ji-1]=rank;
			t=jt-j;
			s += (t*t*t-t);
			j=jt;
		}
	}
	if (j == n) w[n-1]=n;
} 

//---------------------------------------------------------------------------
//
//---------------------------------------------------------------------------
//
//
//
//
//

double betai(const double a, const double b, const double x)
{
	double bt;

	if (x  < 0.0 || x > 1.0)  return(0);
	if (x == 0.0 || x == 1.0) bt=0.0;
	else                      bt=exp(gammln(a+b)-gammln(a)-gammln(b)+a*log(x)+b*log(1.0-x));
	if (x < (a+1.0)/(a+b+2.0))
		return bt*betacf(a,b,x)/a;
	else
		return 1.0-bt*betacf(b,a,1.0-x)/b;
}

//
//
//
//
//

#define DBL_EPSILON 2.2204460492503131e-16
#define DBL_MIN 2.2250738585072014e-308


double betacf(const double a, const double b, const double x)
{
	const int    MAXIT = 100;
	const double EPS   = DBL_EPSILON;
	const double FPMIN = DBL_MIN/EPS;
	int m,m2;
	double aa,c,d,del,h,qab,qam,qap;

	qab=a+b;
	qap=a+1.0;
	qam=a-1.0;
	c=1.0;
	d=1.0-qab*x/qap;
	if (fabs(d) < FPMIN) d=FPMIN;
	d=1.0/d;
	h=d;
	for (m=1;m<=MAXIT;m++) {
		m2=2*m;
		aa=m*(b-m)*x/((qam+m2)*(a+m2));
		d=1.0+aa*d;
		if (fabs(d) < FPMIN) d=FPMIN;
		c=1.0+aa/c;
		if (fabs(c) < FPMIN) c=FPMIN;
		d=1.0/d;
		h *= d*c;
		aa = -(a+m)*(qab+m)*x/((a+m2)*(qap+m2));
		d=1.0+aa*d;
		if (fabs(d) < FPMIN) d=FPMIN;
		c=1.0+aa/c;
		if (fabs(c) < FPMIN) c=FPMIN;
		d=1.0/d;
		del=d*c;
		h *= del;
		if (fabs(del-1.0) <= EPS) break;
	}
//	if (m > MAXIT) nrerror("a or b too big, or MAXIT too small in betacf");
	if (m > MAXIT) return(0);
	return h;
}
//
//
//
//
//

double erfcc(const double  x)
{
	double t,z,ans;

	z = fabs(x);
	t = 1.0/(1.0+0.5*z);
	ans = t*exp(-z*z-1.26551223+t*(1.00002368+t*(0.37409196+t*(0.09678418+
		  t*(-0.18628806+t*(0.27886807+t*(-1.13520398+t*(1.48851587+
		  t*(-0.82215223+t*0.17087277)))))))));
	return (x >= 0.0 ? ans : 2.0-ans);
}
double gammln(const double xx)
{
	static const double cof[6]={ 76.18009172947146,
								-86.50532032941677,
								 24.01409824083091,
								 -1.231739572450155,
								  0.1208650973866179e-2,
							     -0.5395239384953e-5};
	double x,y;

		y = x=xx;
			double tmp=x+5.5; tmp -= (x+0.5)*log(tmp);
			double ser=1.000000000190015;
				for (int j=0; j<6; j++) ser += cof[j]/++y;
	return -tmp+log(2.5066282746310005*ser/x);
} 